#==========================================================================
# 		BOOTSTRAP POT METHODS
#==========================================================================
 getwd()
 source(file.path(WERinR,"Recipes/EVA/EVA-functions.R"))
 Print<-function(x) {cat(x,"\n");  flush.console()}

library(evir)


R=20		# Tne number of epochs for this batch of runs

# Select Weibull indices to be run
ws<-c(0.75,0.9,1,1.1,1.25,1.5,1.75,2,2.25,2.5,2.75,3,3.25,3.5,3.75,4)

wfile<-"Nextw.Rdata"
if (file.exists(wfile))
 {load(wfile)
  if(iw>length(ws)) 
   stop("Bootstrap complete - Consolidate files")
  w<-ws[iw]
  iw<-iw+1
  save(iw,file=wfile)
 }else{
  w<-ws[1]
  iw<-2
  save(iw,file=wfile)
 }

if (!(iw-1)>length(ws))
{#
Print(paste("Bootstrapping POT methods for w =",w))

Ms<-c(50,75,100,125,150,200)		# Number of values over threshold
PIs=c(1.5,2:15)				# Characteristic product
T=1e4						#Number of trials

plotit=FALSE

nM<-length(Ms)
nPI<-length(PIs)
y50<--log(-log(1-1/50))
y100<--log(-log(1-1/100))
y1000<--log(-log(1-1/1000))
y10000<--log(-log(1-1/10000))


boot=function(m,i)
{Z<-data.frame(Uxms=numeric(),Dxms=numeric(),
    Uxmw=numeric(),Dxmw=numeric(),wxmw=numeric(),
    u=numeric(),scl=numeric(),shp=numeric(),
    Ugpd=numeric(),Xmax=numeric(),Gmax=numeric(),
    X50=numeric(),W50=numeric(),G50=numeric(),
    X100=numeric(),W100=numeric(),G100=numeric(),
    X1000=numeric(),W1000=numeric(),G1000=numeric(),
    X10000=numeric(),W10000=numeric(),G10000=numeric())
 Print(paste("M =",Ms[m],"PI =",PIs[i]))
 y<-y.ximis.mean(Ms[m],R)
 P<-1-c(1:Ms[m])/(Ms[m]+1)
 mm<-approx(y,1:Ms[m],0)$y
 Pm<-mm/(Ms[m]+1)
 P50<-1/50*R/Ms[m]
 P100<-1/100*R/Ms[m]
 P1000<-1/1000*R/Ms[m]
 P10000<-1/10000*R/Ms[m]	
 for (t in c(1:T))
 {V<-sort(rximis(Ms[m],R,PIs[i],1,w),decreasing=TRUE)
  u<-min(V)
  xms<-c(summary(pot.XMS(V,R,w))$coefficients[1:2]^(1/w),w)
  Vxms<-qximis(y,xms)
  gpd<-as.numeric(rev(gpd(V,u,method="pwm")$par.ests))  #evir
  ##gpd<-gpd.fit(V,u,npy=Ms[i]/R,show=FALSE)$mle #ismev
  Vgpd<-u+gpd[1]*((1-P)^-gpd[2]-1)/gpd[2]
  Ugpd<-u+gpd[1]*((Pm)^-gpd[2]-1)/gpd[2]
  xmw<-pot.XMW(V,R)$par
  Vxmw<-qximis(y,xmw)
  if (plotit)
  {plot(y,sort(V,decreasing=TRUE))
   lines(y,Vxms,lwd=3)
   lines(y,Vgpd,lwd=3,col="red")
   lines(y,Vxmw,lwd=3,col="blue")
  }
  z<-data.frame(Uxms=xms[1],Dxms=xms[2],
      Uxmw=xmw[1],Dxmw=xmw[2],wxmw=xmw[3],
      u=u,scl=gpd[1],shp=gpd[2],Ugpd=Ugpd,
      Xmax=Vxms[1],Gmax=Vgpd[1],
      X50=qximis(y50,xms),W50=qximis(y50,xmw),
	G50=u+gpd[1]*((P50)^-gpd[2]-1)/gpd[2],
      X100=qximis(y100,xms),W100=qximis(y100,xmw),
	G100=u+gpd[1]*((P100)^-gpd[2]-1)/gpd[2],
      X1000=qximis(y1000,xms),W1000=qximis(y1000,xmw),
	G1000=u+gpd[1]*((P1000)^-gpd[2]-1)/gpd[2],
      X10000=qximis(y10000,xms),W10000=qximis(y10000,xmw),
	G10000=u+gpd[1]*((P10000)^-gpd[2]-1)/gpd[2]
     )
  Z<-rbind(Z,z)
 }
 means<-c(Ms[m],PIs[i],colMeans(Z))
 names(means)<-c("M","PI",colnames(Z))
 variances<-c(Ms[m],PIs[i],apply(Z,2,var))
 names(variances)<-c("M","PI",colnames(Z))
 return(c(means,variances))
}

m<-c(1:nM)
tic<-Sys.time()
Out<-numeric()
for (i in c(1:nPI))
{out<-t(sapply(m,boot,i=i))
 Out<-rbind(Out,out)
}
print(Sys.time()-tic)

Means<-as.data.frame(Out[,1:25])
Variances<-as.data.frame(Out[,26:50])
rownames(Means)<-NULL
rownames(Variances)<-NULL

save(Means,Variances,file=paste("BootData\\PotBoot.w",w,".R",R,".Rdata",sep=""))
}#

#round(Means,4)
#round(Variances,4)



